#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include "../Block.hh"
#include "../Header.hh"
#include "../PSRFits.hh"

using namespace Header;
using namespace Block;
using namespace FITS;
using std::string;

ISMOVING moveStatus(std::string file)
{
  std::ifstream fin;
  fin.open(file.c_str(), std::ios::binary);
  DataBlock block(idSurvey(file));
  block.readBlock(fin);
  fin.close();
  return ISMOVING(block.getKeyword<int>(MOVE));
}

std::string getFileName(int number)
{
  std::stringstream ss;
  ss << "!testfull" << std::setw(2) << std::setfill('0') << number << ".fits";
  return ss.str();
}

int main(int argc, char * argv[])
{
  std::string fname;
  if(argc==1) fname = "data/s70_tape_test_S181_S18101.dat";
  else if(argc==2) fname = argv[1];
  else{
    std::cerr << "Either \"testfulls70.x\" or \"testfulls70.x filename\"...\n";
    return 1;
  }
  //  std::string fname = "../data/pttape_test";
  SURVEY currentSurvey = idSurvey(fname);

  if(currentSurvey == PM){
    std::cerr <<"\aOops! This is a PM survey!\n Good, but not for this program\nExiting...\n"; 
  }
  if(currentSurvey != S70){
    if(currentSurvey!=PM)
      std::cerr <<"\aOops! This isn't an S70 survey file!\nExiting...\n";
    return 1;
  }
  else std::cerr <<"Yes! This is an S70 survey file. Go ahead...\n";

  ISMOVING status = moveStatus(fname);
  if(status == NOTGRID){
    std::cerr <<"\aOops! This is not a gridded observation -- and so not a survey\nExiting...";
    return 1;
  }
  else if(status == SCANNING){
    std::cerr <<"\aOops! This is a scanning observation -- how did that happen?\nExiting...";
    return 1;
  }
  else if(status == MOVING)
    std::cerr <<"OK... we are moving, but carry on in case we stop...\n";
  else if(status == POINT)
    std::cerr <<"Excellent... a pointed observation. Let's go!\n";
  else{
    std::cerr <<"\a\a\aOops! status is an unknown value (" 
	      << status << ")\nExiting...\n";
    return 1;
  }

  std::ifstream fin;
  fin.open(fname.c_str(), std::ios::binary);

  bool eof;
  DataBlock block(currentSurvey);
  block.readBlock(fin);
  
  std::string currentRA = block.getKeyword<std::string>(RA);

  int filecount = 0;
//   PSRFits fitsfile(getFileName(filecount++));
  PSRFits fitsfile;
  if(fitsfile.openFile(block)) {
    std::cerr << "\aCannot open file.\nExiting...\n";
    return 1;
  }
  fitsfile.writePHU(block);
  
  int blockcount = 0;
  do{

    if(block.getKeyword<int>(MOVE)!=MOVING) {
      std::cerr << "block " << blockcount++;
      if(fitsfile.writeSubint(block)){
	std::cerr << "Error. Exiting...\n";
	return 1;
      }
      std::cerr << " done\n";
    }
    eof = !block.readBlock(fin);

    std::cerr << currentRA << " "
	      << block.getKeyword<std::string>(RA) << " "
	      << block.getKeyword<int>(MOVE) <<" ";

    if(!eof){

      if((block.getKeyword<std::string>(RA) != currentRA)
	 || (block.getKeyword<int>(MOVE)==MOVING)){
	std::cerr << "Changing FITS file -- " << filecount << "\n";
	fitsfile.closeFile();
      }

      if((fitsfile.getFilestate()==CLOSED)
	 &&(block.getKeyword<int>(MOVE)==POINT)){
// 	fitsfile.openFile(getFileName(filecount++));
	fitsfile.openFile(block);
	fitsfile.writePHU(block);
	currentRA = block.getKeyword<std::string>(RA);
      }

    }
	
	//work out position
	//    if(!goAgain || position changed) goAgain = false;
	
  }while(!eof);
  
  fitsfile.closeFile();
  fin.close();
  std::cerr << "\n";
}
